#include "RingBuffer.h"

void Buffer_Initialize(RingBuff_t* Buffer)
{
	unsigned char temp_SREG;
    
    //Ensure this is run as an atomic block
    temp_SREG = SREG;//Save the current state of the Global Interrupt bit
    CLI();//Disable global interrupts
    
	Buffer->InPtr    = (RingBuff_Data_t*)&Buffer->Buffer;
	Buffer->OutPtr   = (RingBuff_Data_t*)&Buffer->Buffer;
	Buffer->Elements = 0;
    
    //Restore the Global Interrupts if enabled
    SREG = temp_SREG;
}

void Buffer_StoreElement(RingBuff_t* Buffer, RingBuff_Data_t Data)
{
	unsigned char temp_SREG;
    
    //Ensure this is run as an atomic block
    temp_SREG = SREG;//Save the current state of the Global Interrupt bit
    CLI();//Disable global interrupts
	
	#if defined(BUFF_DROPOLD)
		if (Buffer->Elements == BUFF_LENGTH)
		{
			Buffer->OutPtr++;
	
			if (Buffer->OutPtr == &Buffer->Buffer[BUFF_LENGTH])
			  Buffer->OutPtr = (RingBuff_Data_t*)&Buffer->Buffer;
		}
		else
		{
			Buffer->Elements++;
		}
	#elif defined(BUFF_DROPNEW)
		if (Buffer->Elements == BUFF_LENGTH)
		  return;
	
		  Buffer->Elements++;
	#elif defined(BUFF_NODROPCHECK)
		  Buffer->Elements++;
	#endif
	
		*(Buffer->InPtr) = Data;
		Buffer->InPtr++;
	
		if (Buffer->InPtr == &Buffer->Buffer[BUFF_LENGTH])
		  Buffer->InPtr = (RingBuff_Data_t*)&Buffer->Buffer;
	
    //Restore the Global Interrupts if enabled
    SREG = temp_SREG;
}

RingBuff_Data_t Buffer_GetElement(RingBuff_t* Buffer)
{
	RingBuff_Data_t BuffData;
	
	unsigned char temp_SREG;
    
    //Ensure this is run as an atomic block
    temp_SREG = SREG;//Save the current state of the Global Interrupt bit
    CLI();//Disable global interrupts
    
#if defined(BUFF_EMPTYRETURNSZERO)
		if (!(Buffer->Elements))
		  return 0;
#elif !defined(BUFF_NOEMPTYCHECK)
	#error No empty buffer check behaviour specified.
#endif

		BuffData = *(Buffer->OutPtr);
	
		Buffer->OutPtr++;
		Buffer->Elements--;
	
		if (Buffer->OutPtr == &Buffer->Buffer[BUFF_LENGTH])
		  Buffer->OutPtr = (RingBuff_Data_t*)&Buffer->Buffer;
	
    //Restore the Global Interrupts if enabled
    SREG = temp_SREG;
	
	return BuffData;
}


RingBuff_Data_t Buffer_PeekElement(const RingBuff_t* Buffer)
{
	RingBuff_Data_t BuffData;

	unsigned char temp_SREG;
    
    //Ensure this is run as an atomic block
    temp_SREG = SREG;//Save the current state of the Global Interrupt bit
    CLI();//Disable global interrupts
    
#if defined(BUFF_EMPTYRETURNSZERO)
		if (!(Buffer->Elements))
		  return 0;
#elif !defined(BUFF_NOEMPTYCHECK)
	#error No empty buffer check behaviour specified.
#endif

		BuffData = *(Buffer->OutPtr);
	
    //Restore the Global Interrupts if enabled
    SREG = temp_SREG;

	return BuffData;
}