
/*****************************************************************************
*
* Atmel Corporation

* AppNote           : AVR311 - TWI Slave Implementation
*
* Description       : This is sample driver to AVRs TWI module. 
*                     It is interupt driven.
*
****************************************************************************/

#include <stdio.h>
#include <iom168v.h>
#include "TWI_slave.h"
#include "Board.h"
#include "Serial.h"

#include "RingBuffer.h"
 
static unsigned char TWI_TX_BUF[TWI_TX_BUFFER_SIZE];     // Transceiver buffer. Set the size in the header file
//static unsigned char TWI_RX_BUF[TWI_RX_BUFFER_SIZE];     // Transceiver buffer. Set the size in the header file

static unsigned char TWI_msgSize  = 0;             // Number of bytes to be transmitted.
static unsigned char TWI_state    = TWI_NO_STATE;  // State byte. Default set to TWI_NO_STATE.

unsigned char temp;//=>Billy Try to remove

unsigned char busyTx, busyRx, newData;

unsigned char text[15];


RingBuff_t Rx_Buffer;

char * Print_Int(int intvalue, char *text, unsigned char base);

/****************************************************************************
Call this function to set up the TWI slave to its initial standby state.
Remember to enable interrupts from the main application after initializing the TWI.
Pass both the slave address and the requrements for triggering on a general call in the
same byte. Use e.g. this notation when calling this function:
TWI_Slave_Initialise( (TWI_slaveAddress<<TWI_ADR_BITS) | (TRUE<<TWI_GEN_BIT) );
The TWI module is configured to NACK on any requests. Use a TWI_Start_Transceiver function to 
start the TWI.
****************************************************************************/
void TWI_Slave_Initialise( unsigned char TWI_ownAddress )
{
   
  Buffer_Initialize(&Rx_Buffer);
  
  busyTx = 0;
  busyRx = 0;
  newData = 0;

  TWAR = (TWI_ownAddress<<1) | 0  ;             // Set own TWI slave address. Do not accept TWI General Calls.
  
  //if you want to answer any address
  //TWAMR = 0xFF;
  
  TWDR = 0xFF;                                  // Default content = SDA released.
  
  TWI_state         = TWI_NO_STATE ;
  
  TWCR = (1<<TWEN)|                             // TWI Interface enabled.
         (1<<TWIE)|(1<<TWINT)|                  // Enable TWI Interupt and clear the flag.
         (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|       // Prepare to ACK next time the Slave is addressed.
         (0<<TWWC);
  

}    
    
/****************************************************************************
Call this function to test if the TWI_ISR is busy transmitting.
****************************************************************************/

/*
unsigned char TWI_Transceiver_Busy( void )
{

  return busy;
 // return ( TWCR & (1<<TWIE) );                  // IF TWI interrupt is enabled then the Transceiver is busy
}

*/


/****************************************************************************
Call this function to fetch the state information of the previous operation. The function will hold execution (loop)
until the TWI_ISR has completed with the previous operation. If there was an error, then the function 
will return the TWI State code. 
****************************************************************************/
unsigned char TWI_Get_State_Info( void )
{
  //while ( TWI_Transceiver_Busy() );             // Wait until TWI has completed the transmission.
  return ( TWI_state );                         // Return error state. 
}

/****************************************************************************
Call this function to send a prepared message, or start the Transceiver for reception. Include
a pointer to the data to be sent if a SLA+W is received. The data will be copied to the TWI buffer. 
Also include how many bytes that should be sent. Note that unlike the similar Master function, the
Address byte is not included in the message buffers.
The function will hold execution (loop) until the TWI_ISR has completed with the previous operation,
then initialize the next operation and return.
****************************************************************************/
void TWI_Transceiver_Update_Data( unsigned char *msg, unsigned char msgSize )
{
  	unsigned char temp;
	unsigned char temp_SREG;

  	while ( busyTx ); // Wait until TWI is finished transmitting the current data if it is in the middle of doing so.
 
    //Ensure this is run as an atomic block
    temp_SREG = SREG;//Save the current state of the Global Interrupt bit
    CLI();//Disable global interrupts
  
  	if (msgSize <= TWI_TX_BUFFER_SIZE)
  	{
   	 	TWI_msgSize = msgSize;                        // Number of data to transmit.
  	}
  	else
  	{
   		TWI_msgSize = TWI_TX_BUFFER_SIZE;			   //Only copy what we can
  	}
  
  
  	for ( temp = 0; temp < TWI_msgSize; temp++ )      // Copy data that may be transmitted if the TWI Master requests data.
    {
    	TWI_TX_BUF[ temp ] = msg[ temp ];
    }
        
	//Restore the Global Interrupts if enabled
    SREG = temp_SREG;
}


/****************************************************************************
Call this function to read out the received data from the TWI transceiver buffer. I.e. first call
TWI_Start_Transceiver to get the TWI Transceiver to fetch data. Then Run this function to collect the
data when they have arrived. Include a pointer to where to place the data and the number of bytes
to fetch in the function call. The function will hold execution (loop) until the TWI_ISR has completed 
with the previous operation, before reading out the data and returning.
If there was an error in the previous transmission the function will return the TWI State code.
****************************************************************************/
unsigned char TWI_Get_Data_From_Transceiver( unsigned char *msg, unsigned char msgSize )
{
	unsigned char i = 0;
    unsigned char temp_SREG;

  	while ( busyRx );             // Wait until TWI is ready for next transmission.
 	
    //Ensure this is run as an atomic block
    temp_SREG = SREG;//Save the current state of the Global Interrupt bit
    CLI();//Disable global interrupts
	
    while(Rx_Buffer.Elements && i < msgSize)
    {
    	msg[ i++ ] = Buffer_GetElement(&Rx_Buffer);
    }
    
    
    //Restore the Global Interrupts if enabled
    SREG = temp_SREG;
    
    return i;
  
 // if( newData )               // Last transmission competed successfully.              
  //{
 //   newData = 0;
//    if (msgSize > TWI_RX_BUFFER_SIZE)//if we want to read more data then we can handle
//	{  			  			      //cap it to our buffer size
//	   msgSize = TWI_RX_BUFFER_SIZE;
//	} 
	                                          
//    for ( i=0; i<msgSize; i++ )                 // Copy data from Transceiver buffer.
//    {
//      msg[ i ] = TWI_RX_BUF[ i ];
 //   }
    
	//return 1;
 // }
 // else
//  {
 //  	  return 0;
 // }                                 
}

// ********** Interrupt Handlers ********** //
/****************************************************************************
This function is the Interrupt Service Routine (ISR), and called when the TWI interrupt is triggered;
that is whenever a TWI event has occurred. This function should not be called directly from the main
application.
****************************************************************************/
#pragma interrupt_handler TWI_Interrupt:iv_TWI
void  TWI_Interrupt(void)
{
  static short TWI_bufPtr;
  
  TWI_state = TWSR;//so I know what my last state was (for dubug)

  switch (TWSR)
  {
    case TWI_STX_ADR_ACK: // Own SLA+R has been received; ACK has been returned
//    case TWI_STX_ADR_ACK_M_ARB_LOST: // Arbitration lost in SLA+R/W as Master; own SLA+R has been received; ACK has been returned
      TWI_bufPtr   = 0; // Set buffer pointer to first data location
      busyTx = 1; //This flag says that data is currently being read by the master 
	case TWI_STX_DATA_ACK: // Data byte in TWDR has been transmitted; ACK has been received
 
	  
	  if (TWI_bufPtr < TWI_TX_BUFFER_SIZE)
	  {
	   	 TWDR = TWI_TX_BUF[TWI_bufPtr++];
	  }
	  else
	  {
	   	 TWDR = 0xFF;//no valid data to send 
	  }
	  
	  
      TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to the send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Expect an acknowlege for the byte 
             (0<<TWWC);                         //  
      break;
	  
    case TWI_STX_DATA_NACK: // Data byte in TWDR has been transmitted; NACK has been received. 
                            // I.e. the end of the transmission.

									 
	  busyTx = 0;	// We are no longer being read by the master						 
									 
      TWCR = (1<<TWEN)|                          // Enable TWI-interface
             (1<<TWIE)|(1<<TWINT)|               // Enable Interupt and clear the interrupt
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|    // Acknowledge any new requests.
             (0<<TWWC);                          
      break;
	  
	  
    case TWI_SRX_GEN_ACK:            // General call address has been received; ACK has been returned
//    case TWI_SRX_GEN_ACK_M_ARB_LOST: // Arbitration lost in SLA+R/W as Master; General call address has been received; ACK has been returned

    case TWI_SRX_ADR_ACK:            // Own SLA+W has been received ACK has been returned
//    case TWI_SRX_ADR_ACK_M_ARB_LOST: // Arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned    
                                                        // Dont need to clear TWI_S_statusRegister.generalAddressCall due to that it is the default state.
     
	  busyRx = 1;	// Flag to show that the master is writing to us
	        
    //  TWI_bufPtr   = 0;                           // Set buffer pointer to first data location
                                                  
      TWCR = (1<<TWEN)|                           // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|                // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|     // ACK data
             (0<<TWWC);                           //      
      break;
    case TWI_SRX_ADR_DATA_ACK:       // Previously addressed with own SLA+W; data has been received; ACK has been returned
    case TWI_SRX_GEN_DATA_ACK:       // Previously addressed with general call; data has been received; ACK has been returned
    
    
    Buffer_StoreElement( &Rx_Buffer, TWDR);   
	//  newData = 1; //Flag to say we have new data
	  
	  //Ensure that our buffer has enough room for the data
	//  if (TWI_bufPtr < TWI_RX_BUFFER_SIZE)
	//  {
	 //  	 TWI_RX_BUF[TWI_bufPtr++] = TWDR;
	 // }
	 // else
	 // {
		// temp = TWDR;//read in the unused data
	  //}                      
                                                        
      TWCR = (1<<TWEN)|                             // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|                  // Enable TWI Interupt and clear the flag
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|       // Send ACK after next reception
             (0<<TWWC);                             //  
      break;
    case TWI_SRX_STOP_RESTART:       // A STOP condition or repeated START condition has been received while still addressed as Slave    
		busyRx = 0;
	 
	                                     
      	TWCR = 	(1<<TWEN)|                              // Enable TWI-interface and release TWI pins
             	(1<<TWIE)|(1<<TWINT)|                   // Enable Interupt
             	(1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|        // Acknowledge any new requests.
             	(0<<TWWC); 
	  	break;
	            
    case TWI_SRX_ADR_DATA_NACK:      // Previously addressed with own SLA+W; data has been received; NOT ACK has been returned
    case TWI_SRX_GEN_DATA_NACK:      // Previously addressed with general call; data has been received; NOT ACK has been returned
    
    //Ignore the data
	temp = TWDR;

    case TWI_STX_DATA_ACK_LAST_BYTE: // Last data byte in TWDR has been transmitted (TWEA = 0); ACK has been received
//    case TWI_NO_STATE              // No relevant state information available; TWINT = 0
    case TWI_BUS_ERROR:         // Bus error due to an illegal START or STOP condition
    default:     
      TWI_state = TWSR;                                 // Store TWI State as errormessage, operation also clears the Success bit.      
      uart0_puts ("Error\n");
	  Print_Int(TWI_state,text,16);
	  uart0_puts (text);
	  uart0_puts ("\nHave a nice day");
	  	 
	  TWCR = (1<<TWEN)|                                 // Enable TWI-interface and release TWI pins
             (1<<TWIE)|(1<<TWINT)|                      // Ensable Interupt
             (1<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|           // Acknewlege any new requests. Release the SDA SCL lines
             (0<<TWWC);                                 //
  }
}


char * Print_Int(int intvalue, char *text, unsigned char base)
{
	 char myLetters[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	 int start=0, count = 0;
	 unsigned char temp;
	 
	 if (base == 10)
	 {
		 if(intvalue < 0)
		 {	 
			text[count++] = '-';
			intvalue = -intvalue;
			start++;
		 } 
	 }else if (base == 16)
	 {	 
		text[count++] = '0';
		text[count++] = 'X';
		start+=2;	 
	 } else if (base == 8)
	 {	 
		text[count++] = '0';
		text[count++] = 'O';
		start+=2;	 
	 } else if (base == 2)
	 {	 
		text[count++] = '0';
		text[count++] = 'B';
		start+=2;	 
	 } 
	 
	 //find the digits
	 do {
	 	text[count++] = myLetters[intvalue % base];
	 } while ((intvalue /= base) > 0);
	 
	 text[count--] = 0x00;
	 
	 //reverse to correct order
	 while (start < count)
	 {
		 temp = text[start];
		 text[start] = text[count];
		 text[count] = temp;
		 start++;
		 count--; 
	 }
	 
	 return text;
	 
}