/**************************************************************

   Program Title:  Led8 Controller
   
   Program Name:  LED8_Init.c		  Version: 0.85
   
   Author:  SS	  					  Date:  August 10, 2005
   
   Compiled with ImageCraft ICCAVR Standard V7.02
   
   This file contains routines that implement the basic initialization 
   routines for the Wasp AVR Monitor.
   
   Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC MACHINES, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Wasp_Monitor.c,v $
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom48v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
//#include <stdio.h>
#include "LED8_Uart.h"
#include "LED8_Defines.h"
#include "LED8_Extern.h"

//****************************************************************
//****************************************************************
// Initialization code
//****************************************************************
//****************************************************************

// Initialize UART
void InitUART(unsigned char baudrate)
{
	unsigned char x;
	
	// Set the baud rate
	UBRR0H = 0x00;
	UBRR0L = baudrate; 
	// Enable UART receiver and transmitter
	UCSR0B = BIT(RXCIE0)|BIT(UDRIE0)|BIT(RXEN0)|BIT(TXEN0); 
    //  8 data bits, 2 stop bits
//	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
    //  8 data bits, 1 stop bit
	UCSR0C = (1<<UMSEL01)|(3<<UCSZ01);

	x = 0; // flush receive buffer
	rxcnt = x; // rx buffer empty
	xoffsent = NO;
	sendxoff = NO;
	xonsent = NO;
	checkflag = NO;
	enablexonoff = YES; // turn serial driver xon/xoff flow control on
	echoinchars = NO;   // echo all received characters
	UART_RxTail = x;
	UART_RxHead = x;
	UART_TxTail = x;
	UART_TxHead = x;
	dataoverrun = NO;
	frameerror = NO;
	xoncnt = 0;
	xoffcnt = 0;
}

//  Setup the Analog to Digital converter
//  TBD - convert to background interrupt process
void InitADC(void)
{
	DDRB = 0x00;  // All inputs
		   		  // Setup Analog TDB
}

//  Start the real time clock 
//  Clocked by external 32.768KHz watch crystal
//  Set prescalar to 0, set to interrupt using compare match 
//  set to 128 (4msec interrupt rate).  
void InitRealTimeClock(void)
{  
    // Software counters - approximately 4msec resolution
	rtccountdown1 = 0;  
	rtccountdown2 = 0;
	rtccountdown3 = 0;
	rtccountdown4 = 0;
	
 	TCNT2 = 0;
 	// Select external real time clock oscillator
	ASSR = (1<<AS2);
	// Set match interrupt to 4msec
	OCR2A = 127;
	// Reload counter on match interrupt - pre-scalar set to 0
	TCCR2A = (1<<WGM21)|(1<<CS20);   
    // setup interrupts for real time clock
    TIMSK2 |= (1<<OCIE2A);
}

// Timer2 is called every 1msec - used to maintain four count down 
// software timers to control precise time delays in various routines
void Init_Timer0(void)
{
 	 mseccountdown1 = 0;
	 mseccountdown2 = 0;
	 mseccountdown3 = 0;
	 
	 ledindex = 0;
	 
	 //PRR = (PRR&0xDF);
	 
	 TCNT0 = 0;
	 // Set mode 2 - reset counter on compare match, clk/8
	 TCCR0B = ((1<<CS01)|(1<<CS00));
	 TCCR0A = (1<<WGM01);
	 // Reset count after 1 msec
	 OCR0A = 252;
	 // Enable compare match interrupt vector
	 TIMSK0 |= (1<<OCIE0A);
}

// Setup watch dog timer - TBD
void InitWatchdog(void)
{
}

void Init_SPI_Master(void)
{
    // Set MOSI and SCK output, all others input
	DDRB = (1<<PB5)|(1<<PB7)|(1<<PB4);
	PORTB |= 0xA0;  // Set bits MOSI & SCK

	//  Make Flash Chip select PC4 output and set high
	DDRC |= 0x10; 
	PORTC |= 0x10;  
	
	// Enable SPI, Master, set clock rate fck
	//    Select SPI Mode 3 - CPOL=1, CPHA=1
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPOL)|(1<<CPHA)|(1<<SPR0);
//	SPCR = 0x91;
}

//************************************************************
//  Initialize the AVR - Set LED start conditions
//************************************************************
void Init_AVR(void)
{
	// Setup Port D
	//   RxD - PD0, TxD - PD1
	//   LED1-PD2, LED2-PD3, LED3-PD4, LED4-PD5, LED5-PD6, LED6-PD7
	DDRD = 0xFC;
	PORTD = 0x00;  // Turn all LED master power enables off
	
	// Setup Port B
	//   LED7-PB6, LED8-PB7, DP-PB2,  PC6-g
	//   New LED8 - LED7-PB5, LED8-PB4, DP-PB2,  PB1-e, PB3-f, PB0-g
	DDRB = 0x3F;
	PORTB = 0x00;  // Turn LED power enables off
	
	// Setup Port C - LED Character control
	//   Bit PC0 - a, PC1-b, PC2-c, PC3-d, PC4-SDA, PC5-SCL
	//   Bit PC7 - Reset
	DDRC = 0x7F;
	PORTC = 0x7F;  // turn all LED segments off
	
	//   Setup SPI - SPI default inactive in this release
//    Init_SPI_Master();
	
	//    Turn on real time clock on PC7,6 Timer/Counter 2
//	InitRealTimeClock();
	
	// Setup Timer 0 to interrupt a 1msec rate - this interrupt
	// controls four software delay timers used by various 
	// routines
	Init_Timer0();
	
	//    Setup 2-Wire Interface
	//Init_TWI_Master();   // SCL freq = sclk/(16+2*TWBR)*4*TWPS

	// Watchdog timer setup
//	InitWatchdog();
	
	// Initialize UART - Interrupt
	// Set the baudrate to 9,600 bps using an 8MHz clock
//	InitUART(BAUDRATE9600); 
					
}
