/**************************************************************

   Program Title:  Led8 Controller
   
   Program Name:  LED8_Utilities.c	  Version: 0.85
   
   Author:  SS	  					  Date:  August 10, 2005
   
   Compiled with ImageCraft ICCAVR Standard V7.02
   
   This file contains routines that implement a set of basic utility 
   functions for the Wasp Monitor.
   
   Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC MACHINES, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Wasp_Monitor.c,v $
 * Revision 0.86  2006/02/07  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom48v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
//#include <stdio.h>
#include "LED8_Uart.h"
#include "LED8_Defines.h"
#include "LED8_Extern.h"

//*********************************************************
//*********************************************************
//      Utilities
//*********************************************************
//*********************************************************
void Delay(void)
{
	unsigned char a, b;

	for (a = 1; a; a++)
		for (b = 1; b; b++)
			;
}

void ShortDelay(int cnt)
{
 	unsigned char inbyte;
	int i;
	
    for(i=0; i<cnt; i++)  {
	    inbyte = 0x55;
		}
}

void LongDelay(int cnt)
{
 	 while(cnt > 0) {
	 	 Delay();
		 cnt--;
		 }
}

void Delaymsec(int delayvalue)
{
 	 mseccountdown2 = delayvalue;
	 while(mseccountdown2) ;
}

// Function to initiate an EEPROM write
// writes the specified data byte to the specified location
// this will fail if the EEPROM is not ready!
void RTEEPROMwrite(int location, unsigned char databyte)
{
	unsigned char savedSREG;
	EEAR = location;								// set address
	EEDR = databyte;								// set data	
	savedSREG = SREG;								// keep setting so it can be restored
	CLI();													// disable interrupts
	EECR |= BIT(EEMWE);							// set "write enable" bit
	EECR |= BIT(EEWE);							// set "write" bit
	SREG = savedSREG;								// restore SREG
	EEAR = 0;
}

// EEPROM writes take about 8ms - so poll until done
// Do not use ICC V7 function EEPROMwrite() - this routine turns interrupts off!
void EEPROMWrite(int location, unsigned char databyte)
{
 	 while(!RTEEPROMReady())  ;
 	 RTEEPROMwrite(location,databyte);
 	 while(!RTEEPROMReady())  ;
}

unsigned char hextobyte(unsigned char inbyte)
{
	 if((inbyte >= 'A')&(inbyte<='F')) return (inbyte - 'A' + 10);
	 if((inbyte >= 'a')&(inbyte<='f')) return (inbyte - 'a' + 10);
	 else return (inbyte - '0');
}

// function to test if the EEPROM is ready for a read or write operation
// returns non zero if ready, zero if not ready 
unsigned char RTEEPROMReady(void)
{
	return !(EECR & 0x02);
}

void LED_Power(int ledselect,char state)
{
 	 if(ledselect==LED1) {
	 	if(state==ON)  SetBit(PORTD,PIND2);
		else  ClrBit(PORTD,PIND2);
		}
 	 else if(ledselect==LED2) {
	 	if(state==ON)  SetBit(PORTD,PIND3);
		else  ClrBit(PORTD,PIND3);
		}
 	 else if(ledselect==LED3) {
	 	if(state==ON)  SetBit(PORTD,PIND4);
		else  ClrBit(PORTD,PIND4);
		}
 	 else if(ledselect==LED4) {
	 	if(state==ON)  SetBit(PORTD,PIND5);
		else  ClrBit(PORTD,PIND5);
		}
 	 else if(ledselect==LED5) {
	 	if(state==ON)  SetBit(PORTD,PIND6);
		else  ClrBit(PORTD,PIND6);
		}
 	 else if(ledselect==LED6) {
	 	if(state==ON)  SetBit(PORTD,PIND7);
		else  ClrBit(PORTD,PIND7);
		}
// 	 New PCB
	 else if(ledselect==LED7) {
	 	if(state==ON)  SetBit(PORTB,PINB5);
		else  ClrBit(PORTB,PINB5);
		}
 	 else if(ledselect==LED8) {
	 	if(state==ON)  SetBit(PORTB,PINB4);
		else  ClrBit(PORTB,PINB4);
		} 

/*   Old PCB
 	 else if(ledselect==LED7) {
	 	if(state==ON)  SetBit(PORTB,PINB6);
		else  ClrBit(PORTB,PINB6);
		}
 	 else if(ledselect==LED8) {
	 	if(state==ON)  SetBit(PORTB,PINB7);
		else  ClrBit(PORTB,PINB7);
		}
*/
}

//  Use PortB bit 0
void  Set_LED_Segments(unsigned char ledbyte)
{
	 if(ledbyte&0x01) SetBit(PORTC,PINC0);
	 else ClrBit(PORTC,PINC0);
	 if(ledbyte&0x02) SetBit(PORTC,PINC1);
	 else ClrBit(PORTC,PINC1);
	 if(ledbyte&0x04) SetBit(PORTC,PINC2);
	 else ClrBit(PORTC,PINC2);
	 if(ledbyte&0x08) SetBit(PORTC,PINC3);
	 else ClrBit(PORTC,PINC3);
	 if(ledbyte&0x10) SetBit(PORTB,PINB1);
	 else ClrBit(PORTB,PINB1);
	 if(ledbyte&0x20) SetBit(PORTB,PINB3);
	 else ClrBit(PORTB,PINB3);
	 if(ledbyte&0x40) SetBit(PORTB,PINB0);
	 else ClrBit(PORTB,PINB0);
}

void  Decimal_Point(char state)
{
	 if(state==ON)  ClrBit(PORTB,PINB2);
	 else  SetBit(PORTB,PINB2);
}

void  Cycle_LED(int ledselect)
{
 	int i;
	
	LED_Power(ledselect,ON);
	for(i=0; i<10; i++)  {
		 Set_LED_Segments(~ledchars[i]);
		 ShortDelay(10000);
		 }
	LED_Power(ledselect,OFF);
}


// Display number as fast as possible
void Display_Number_Fast(char *numstrg,int decimalpoint)
{
 	 int delaynum;
	 
	 delaynum = 500;
	 Set_LED_Segments(~ledchars[numstrg[0]]);
 	 PORTD = LED1ON;
	 if(decimalpoint==7) Decimal_Point(ON);
	 else  Decimal_Point(OFF);
	 ShortDelay(delaynum);
	 Set_LED_Segments(~ledchars[numstrg[1]]);
 	 PORTD = LED2ON;
	 if(decimalpoint==6) Decimal_Point(ON);
	 else  Decimal_Point(OFF);
	 ShortDelay(delaynum);
 	 LED_Power(LED2,OFF);
	 Set_LED_Segments(~ledchars[numstrg[2]]);
 	 PORTD = LED3ON;
	 if(decimalpoint==5) Decimal_Point(ON);
	 else  Decimal_Point(OFF);
	 ShortDelay(delaynum);
	 Set_LED_Segments(~ledchars[numstrg[3]]);
 	 PORTD = LED4ON;
	 if(decimalpoint==4) Decimal_Point(ON);
	 else  Decimal_Point(OFF);
	 ShortDelay(delaynum);
	 Set_LED_Segments(~ledchars[numstrg[4]]);
 	 PORTD = LED5ON;
	 if(decimalpoint==3) Decimal_Point(ON);
	 else  Decimal_Point(OFF);
	 ShortDelay(delaynum);
	 Set_LED_Segments(~ledchars[numstrg[5]]);
 	 PORTD = LED6ON;
	 if(decimalpoint==2) Decimal_Point(ON);
	 else  Decimal_Point(OFF);
	 ShortDelay(delaynum);
	 PORTD = LEDSOFF;
	 Set_LED_Segments(~ledchars[numstrg[6]]);
 	 PORTB |= LED7ON;
	 if(decimalpoint==1) Decimal_Point(ON);
	 else  Decimal_Point(OFF);
	 ShortDelay(delaynum);
	 Set_LED_Segments(~ledchars[numstrg[7]]);
	 PORTB &= ~LED7ON;
 	 PORTB |= LED8ON;
	 if(decimalpoint==0) Decimal_Point(ON);
	 else  Decimal_Point(OFF);
	 ShortDelay(delaynum);
	 PORTB &= ~LED8ON;
	 Set_LED_Segments(0xff);
}
